@@ -76,9 +76,6 @@ struct _GsmAutostartAppPrivate {
         int                   launch_type;
         GPid                  pid;
         guint                 child_watch_id;
-
-        DBusGProxy           *proxy;
-        DBusGProxyCall       *proxy_call;
 };
 
 enum {
@@ -114,7 +111,6 @@ static gboolean
 is_disabled (GsmApp *app)
 {
         GsmAutostartAppPrivate *priv;
-        const char *current_desktop;
 
         priv = GSM_AUTOSTART_APP (app)->priv;
 
@@ -136,13 +132,9 @@ is_disabled (GsmApp *app)
         }
 
         /* Check OnlyShowIn/NotShowIn/TryExec */
-        current_desktop = gsm_util_get_current_desktop ();
-        g_desktop_app_info_set_desktop_env (current_desktop);
-        if (current_desktop != NULL &&
-            !g_desktop_app_info_get_show_in (G_DESKTOP_APP_INFO (priv->app_info),
-                                             current_desktop)) {
-                        g_debug ("app %s not for %s",
-                                 gsm_app_peek_id (app), current_desktop);
+        if (!g_desktop_app_info_get_show_in (priv->app_info, NULL)) {
+                g_debug ("app %s is not for the current desktop",
+                         gsm_app_peek_id (app));
                 return TRUE;
         }
 
@@ -342,12 +334,16 @@ setup_gsettings_condition_monitor (GsmAutostartApp *app,
         GSettingsSchemaSource *source;
         GSettingsSchema *schema;
         GSettings *settings;
+        GSettingsSchemaKey *schema_key;
+        const GVariantType *key_type;
         char **elems;
         gboolean retval = FALSE;
         char *signal;
 
         retval = FALSE;
 
+        schema = NULL;
+
         elems = g_strsplit (key, " ", 2);
 
         if (elems == NULL)
@@ -363,9 +359,24 @@ setup_gsettings_condition_monitor (GsmAutostartApp *app,
         if (schema == NULL)
                 goto out;
 
+        if (!g_settings_schema_has_key (schema, elems[1]))
+                goto out;
+
+        schema_key = g_settings_schema_get_key (schema, elems[1]);
+
+        g_assert (schema_key != NULL);
+
+        key_type = g_settings_schema_key_get_value_type (schema_key);
+
+        g_settings_schema_key_unref (schema_key);
+
+        g_assert (key_type != NULL);
+
+        if (!g_variant_type_equal (key_type, G_VARIANT_TYPE_BOOLEAN))
+                goto out;
+
         settings = g_settings_new_full (schema, NULL, NULL);
         retval = g_settings_get_boolean (settings, elems[1]);
-        g_settings_schema_unref (schema);
 
         signal = g_strdup_printf ("changed::%s", elems[1]);
         g_signal_connect (G_OBJECT (settings), signal,
@@ -375,6 +386,8 @@ setup_gsettings_condition_monitor (GsmAutostartApp *app,
         app->priv->condition_settings = settings;
 
 out:
+        if (schema)
+                g_settings_schema_unref (schema);
         g_strfreev (elems);
 
         return retval;
@@ -761,13 +774,6 @@ gsm_autostart_app_dispose (GObject *object)
                 priv->child_watch_id = 0;
         }
 
-        if (priv->proxy_call != NULL) {
-                dbus_g_proxy_cancel_call (priv->proxy, priv->proxy_call);
-                priv->proxy_call = NULL;
-        }
-
-        g_clear_object (&priv->proxy);
-
         if (priv->condition_monitor) {
                 g_file_monitor_cancel (priv->condition_monitor);
         }
@@ -1056,21 +1062,19 @@ autostart_app_start_spawn (GsmAutostartApp *app,
 }
 
 static void
-start_notify (DBusGProxy      *proxy,
-              DBusGProxyCall  *call,
-              GsmAutostartApp *app)
+start_notify (GObject      *source,
+              GAsyncResult *result,
+              gpointer      user_data)
 {
-        gboolean res;
-        GError  *error;
+        GError          *error;
+        GsmAutostartApp *app;
 
+        app  = user_data;
         error = NULL;
-        res = dbus_g_proxy_end_call (proxy,
-                                     call,
-                                     &error,
-                                     G_TYPE_INVALID);
-        app->priv->proxy_call = NULL;
 
-        if (! res) {
+        g_dbus_connection_call_finish (G_DBUS_CONNECTION (source), result, &error);
+
+        if (error != NULL) {
                 g_warning ("GsmAutostartApp: Error starting application: %s", error->message);
                 g_error_free (error);
         } else {
@@ -1085,15 +1089,13 @@ autostart_app_start_activate (GsmAutostartApp  *app,
         const char      *name;
         char            *path;
         char            *arguments;
-        DBusGConnection *bus;
+        GDBusConnection *bus;
         GError          *local_error;
 
         local_error = NULL;
-        bus = dbus_g_bus_get (DBUS_BUS_SESSION, &local_error);
-        if (bus == NULL) {
-                if (local_error != NULL) {
-                        g_warning ("error getting session bus: %s", local_error->message);
-                }
+        bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &local_error);
+        if (local_error != NULL) {
+                g_warning ("error getting session bus: %s", local_error->message);
                 g_propagate_error (error, local_error);
                 return FALSE;
         }
@@ -1111,33 +1113,17 @@ autostart_app_start_activate (GsmAutostartApp  *app,
         arguments = g_desktop_app_info_get_string (app->priv->app_info,
                                                    GSM_AUTOSTART_APP_DBUS_ARGS_KEY);
 
-        app->priv->proxy = dbus_g_proxy_new_for_name (bus,
-                                                      name,
-                                                      path,
-                                                      GSM_SESSION_CLIENT_DBUS_INTERFACE);
-        if (app->priv->proxy == NULL) {
-                g_set_error (error,
-                             GSM_APP_ERROR,
-                             GSM_APP_ERROR_START,
-                             "Unable to start application: unable to create proxy for client");
-                return FALSE;
-        }
-
-        app->priv->proxy_call = dbus_g_proxy_begin_call (app->priv->proxy,
-                                                         "Start",
-                                                         (DBusGProxyCallNotify)start_notify,
-                                                         app,
-                                                         NULL,
-                                                         G_TYPE_STRING, arguments,
-                                                         G_TYPE_INVALID);
-        if (app->priv->proxy_call == NULL) {
-                g_clear_object (&app->priv->proxy);
-                g_set_error (error,
-                             GSM_APP_ERROR,
-                             GSM_APP_ERROR_START,
-                             "Unable to start application: unable to call Start on client");
-                return FALSE;
-        }
+        g_dbus_connection_call (bus,
+                                name,
+                                path,
+                                GSM_SESSION_CLIENT_DBUS_INTERFACE,
+                                "Start",
+                                g_variant_new ("(s)", arguments),
+                                NULL,
+                                G_DBUS_CALL_FLAGS_NONE,
+                                -1, NULL,
+                                start_notify, app);
+        g_object_unref (bus);
 
         return TRUE;
 }
