@@ -28,13 +28,8 @@
 
 #include <glib/gi18n.h>
 #include <glib.h>
-
 #include <glib-unix.h>
-
-#include <dbus/dbus.h>
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-bindings.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#include <gio/gio.h>
 
 #include "gdm-log.h"
 
@@ -55,13 +50,12 @@ static gboolean failsafe = FALSE;
 static gboolean show_version = FALSE;
 static gboolean debug = FALSE;
 static gboolean please_fail = FALSE;
-
-static DBusGProxy *bus_proxy = NULL;
+static gboolean disable_acceleration_check = FALSE;
+static const char *session_name = NULL;
+static GsmManager *manager = NULL;
 
 static GMainLoop *loop;
 
-static void shutdown_cb (gpointer data);
-
 void
 gsm_quit (void)
 {
@@ -78,111 +72,28 @@ gsm_main (void)
 }
 
 static void
-on_bus_name_lost (DBusGProxy *bus_proxy,
-                  const char *name,
-                  gpointer    data)
+on_name_lost (GDBusConnection *connection,
+              const char *name,
+              gpointer    data)
 {
-        g_warning ("Lost name on bus: %s, exiting", name);
-        exit (1);
-}
+        GsmManager *manager = (GsmManager *)data;
 
-static gboolean
-acquire_name_on_proxy (DBusGProxy *bus_proxy,
-                       const char *name)
-{
-        GError     *error;
-        guint       result;
-        gboolean    res;
-        gboolean    ret;
-
-        ret = FALSE;
-
-        if (bus_proxy == NULL) {
-                goto out;
-        }
-
-        error = NULL;
-        res = dbus_g_proxy_call (bus_proxy,
-                                 "RequestName",
-                                 &error,
-                                 G_TYPE_STRING, name,
-                                 G_TYPE_UINT, 0,
-                                 G_TYPE_INVALID,
-                                 G_TYPE_UINT, &result,
-                                 G_TYPE_INVALID);
-        if (! res) {
-                if (error != NULL) {
-                        g_warning ("Failed to acquire %s: %s", name, error->message);
-                        g_error_free (error);
-                } else {
-                        g_warning ("Failed to acquire %s", name);
-                }
-                goto out;
-        }
-
-        if (result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-                if (error != NULL) {
-                        g_warning ("Failed to acquire %s: %s", name, error->message);
-                        g_error_free (error);
-                } else {
-                        g_warning ("Failed to acquire %s", name);
-                }
-                goto out;
-        }
-
-        /* register for name lost */
-        dbus_g_proxy_add_signal (bus_proxy,
-                                 "NameLost",
-                                 G_TYPE_STRING,
-                                 G_TYPE_INVALID);
-        dbus_g_proxy_connect_signal (bus_proxy,
-                                     "NameLost",
-                                     G_CALLBACK (on_bus_name_lost),
-                                     NULL,
-                                     NULL);
-
-
-        ret = TRUE;
-
- out:
-        return ret;
-}
-
-static gboolean
-acquire_name (void)
-{
-        GError          *error;
-        DBusGConnection *connection;
-
-        error = NULL;
-        connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
         if (connection == NULL) {
-                gsm_util_init_error (TRUE,
-                                     "Could not connect to session bus: %s",
-                                     error->message);
-                return FALSE;
-        }
+                g_warning ("Lost name on bus: %s", name);
+                gsm_fail_whale_dialog_we_failed (TRUE, TRUE, NULL);
+        } else {
+                g_debug ("Calling name lost callback function");
 
-        bus_proxy = dbus_g_proxy_new_for_name_owner (connection,
-                                                     DBUS_SERVICE_DBUS,
-                                                     DBUS_PATH_DBUS,
-                                                     DBUS_INTERFACE_DBUS,
-                                                     &error);
-        if (error != NULL) {
-                gsm_util_init_error (TRUE,
-                                     "Could not connect to session bus: %s",
-                                     error->message);
-                return FALSE;
-        }
+                /*
+                 * When the signal handler gets a shutdown signal, it calls
+                 * this function to inform GsmManager to not restart
+                 * applications in the off chance a handler is already queued
+                 * to dispatch following the below call to gtk_main_quit.
+                 */
+                gsm_manager_set_phase (manager, GSM_MANAGER_PHASE_EXIT);
 
-        if (! acquire_name_on_proxy (bus_proxy, GSM_DBUS_NAME) ) {
-                gsm_util_init_error (TRUE,
-                                     "%s",
-                                     "Could not acquire name on session bus");
-                return FALSE;
+                gsm_quit ();
         }
-
-        return TRUE;
 }
 
 static gboolean
@@ -213,20 +124,48 @@ sigusr1_cb (gpointer data)
 }
 
 static void
-shutdown_cb (gpointer data)
+create_manager (void)
 {
-        GsmManager *manager = (GsmManager *)data;
-        g_debug ("Calling shutdown callback function");
+        GsmStore *client_store;
 
-        /*
-         * When the signal handler gets a shutdown signal, it calls
-         * this function to inform GsmManager to not restart
-         * applications in the off chance a handler is already queued
-         * to dispatch following the below call to gtk_main_quit.
-         */
-        gsm_manager_set_phase (manager, GSM_MANAGER_PHASE_EXIT);
+        client_store = gsm_store_new ();
+        manager = gsm_manager_new (client_store, failsafe);
+        g_object_unref (client_store);
 
-        gsm_quit ();
+        g_unix_signal_add (SIGTERM, term_or_int_signal_cb, manager);
+        g_unix_signal_add (SIGINT, term_or_int_signal_cb, manager);
+        g_unix_signal_add (SIGUSR1, sigusr1_cb, manager);
+        g_unix_signal_add (SIGUSR2, sigusr2_cb, manager);
+
+        if (IS_STRING_EMPTY (session_name)) {
+                session_name = _gsm_manager_get_default_session (manager);
+        }
+
+        if (!gsm_session_fill (manager, session_name)) {
+                gsm_fail_whale_dialog_we_failed (FALSE, TRUE, NULL);
+        }
+
+        gsm_manager_start (manager);
+}
+
+static void
+on_bus_acquired (GDBusConnection *connection,
+                 const char *name,
+                 gpointer data)
+{
+        create_manager ();
+}
+
+static guint
+acquire_name (void)
+{
+        return g_bus_own_name (G_BUS_TYPE_SESSION,
+                               GSM_DBUS_NAME,
+                               G_BUS_NAME_OWNER_FLAGS_NONE,
+                               on_bus_acquired,
+                               NULL,
+                               on_name_lost,
+                               NULL, NULL);
 }
 
 static gboolean
@@ -253,7 +192,7 @@ require_dbus_session (int      argc,
         new_argv[1] = "--exit-with-session";
         for (i = 0; i < argc; i++) {
                 new_argv[i + 2] = argv[i];
-	}
+        }
         new_argv[i + 2] = NULL;
         
         if (!execvp ("dbus-launch", new_argv)) {
@@ -292,12 +231,10 @@ int
 main (int argc, char **argv)
 {
         GError           *error = NULL;
-        GsmManager       *manager;
-        GsmStore         *client_store;
         static char     **override_autostart_dirs = NULL;
         static char      *opt_session_name = NULL;
-        const char       *session_name;
         gboolean          gl_failed = FALSE;
+        guint             name_owner_id;
         GOptionContext   *options;
         static GOptionEntry entries[] = {
                 { "autostart", 'a', 0, G_OPTION_ARG_STRING_ARRAY, &override_autostart_dirs, N_("Override standard autostart directories"), N_("AUTOSTART_DIR") },
@@ -307,6 +244,7 @@ main (int argc, char **argv)
                 { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL },
                 /* Translators: the 'fail whale' is the black dialog we show when something goes seriously wrong */
                 { "whale", 0, 0, G_OPTION_ARG_NONE, &please_fail, N_("Show the fail whale dialog for testing"), NULL },
+                { "disable-acceleration-check", 0, 0, G_OPTION_ARG_NONE, &disable_acceleration_check, N_("Disable hardware acceleration check"), NULL },
                 { NULL, 0, 0, 0, NULL, NULL, NULL }
         };
 
@@ -315,25 +253,6 @@ main (int argc, char **argv)
                 gsm_util_init_error (TRUE, "%s", error->message);
         }
 
-        /* Check GL, if it doesn't work out then force software fallback */
-        if (!check_gl (&error)) {
-                gl_failed = TRUE;
-
-                g_debug ("hardware acceleration check failed: %s",
-                         error? error->message : "");
-                g_clear_error (&error);
-                if (g_getenv ("LIBGL_ALWAYS_SOFTWARE") == NULL) {
-                        g_setenv ("LIBGL_ALWAYS_SOFTWARE", "1", TRUE);
-                        if (!check_gl (&error)) {
-                                g_warning ("software acceleration check failed: %s",
-                                           error? error->message : "");
-                                g_clear_error (&error);
-                        } else {
-                                gl_failed = FALSE;
-                        }
-                }
-        }
-
         bindtextdomain (GETTEXT_PACKAGE, LOCALE_DIR);
         bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
         textdomain (GETTEXT_PACKAGE);
@@ -349,11 +268,6 @@ main (int argc, char **argv)
 
         g_option_context_free (options);
 
-        if (show_version) {
-                g_print ("%s %s\n", argv [0], VERSION);
-                exit (0);
-        }
-
         /* Rebind stdout/stderr to the journal explicitly, so that
          * journald picks ups the nicer "gnome-session" as the program
          * name instead of whatever shell script GDM happened to use.
@@ -370,6 +284,37 @@ main (int argc, char **argv)
         }
 #endif
 
+        gdm_log_init ();
+        gdm_log_set_debug (debug);
+
+        if (disable_acceleration_check) {
+                g_debug ("hardware acceleration check is disabled");
+        } else {
+                /* Check GL, if it doesn't work out then force software fallback */
+                if (!check_gl (&error)) {
+                        gl_failed = TRUE;
+
+                        g_debug ("hardware acceleration check failed: %s",
+                                 error? error->message : "");
+                        g_clear_error (&error);
+                        if (g_getenv ("LIBGL_ALWAYS_SOFTWARE") == NULL) {
+                                g_setenv ("LIBGL_ALWAYS_SOFTWARE", "1", TRUE);
+                                if (!check_gl (&error)) {
+                                        g_warning ("software acceleration check failed: %s",
+                                                   error? error->message : "");
+                                        g_clear_error (&error);
+                                } else {
+                                        gl_failed = FALSE;
+                                }
+                        }
+                }
+        }
+
+        if (show_version) {
+                g_print ("%s %s\n", argv [0], VERSION);
+                exit (0);
+        }
+
         if (gl_failed) {
                 gsm_fail_whale_dialog_we_failed (FALSE, TRUE, NULL);
                 gsm_main ();
@@ -382,9 +327,6 @@ main (int argc, char **argv)
                 exit (1);
         }
 
-        gdm_log_init ();
-        gdm_log_set_debug (debug);
-
         /* From 3.14 GDM sets XDG_CURRENT_DESKTOP. For compatibility with
          * older versions of GDM,  other display managers, and startx,
          * set a fallback value if we don't find it set.
@@ -401,7 +343,8 @@ main (int argc, char **argv)
          */
         gsm_util_setenv ("XDG_MENU_PREFIX", "gnome-");
 
-        client_store = gsm_store_new ();
+        gsm_util_set_autostart_dirs (override_autostart_dirs);
+        session_name = opt_session_name;
 
         /* Talk to logind before acquiring a name, since it does synchronous
          * calls at initialization time that invoke a main loop and if we
@@ -410,44 +353,13 @@ main (int argc, char **argv)
          */
         g_object_unref (gsm_get_system ());
 
-        if (!acquire_name ()) {
-                gsm_fail_whale_dialog_we_failed (TRUE, TRUE, NULL);
-                gsm_main ();
-                exit (1);
-        }
-
-        manager = gsm_manager_new (client_store, failsafe);
-
-        g_signal_connect_object (bus_proxy,
-                                 "destroy",
-                                 G_CALLBACK (shutdown_cb),
-                                 manager,
-                                 G_CONNECT_SWAPPED);
-
-        g_unix_signal_add (SIGTERM, term_or_int_signal_cb, manager);
-        g_unix_signal_add (SIGINT, term_or_int_signal_cb, manager);
-        g_unix_signal_add (SIGUSR1, sigusr1_cb, manager);
-        g_unix_signal_add (SIGUSR2, sigusr2_cb, manager);
-
-        if (IS_STRING_EMPTY (opt_session_name))
-                session_name = _gsm_manager_get_default_session (manager);
-        else
-                session_name = opt_session_name;
-
-        gsm_util_set_autostart_dirs (override_autostart_dirs);
-
-        if (!gsm_session_fill (manager, session_name)) {
-                gsm_fail_whale_dialog_we_failed (FALSE, TRUE, NULL);
-        }
-
-        gsm_manager_start (manager);
+        name_owner_id  = acquire_name ();
 
         gsm_main ();
 
         g_clear_object (&manager);
-        g_clear_object (&client_store);
-        g_clear_object (&bus_proxy);
 
+        g_bus_unown_name (name_owner_id);
         gdm_log_shutdown ();
 
         return 0;
